@@ -105,9 +105,9 @@ destroy_and_quit(VteTerminal *terminal, GtkWidget *window)
 		stream = G_OUTPUT_STREAM (g_file_replace (file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error));
 
 		if (stream) {
-			vte_terminal_write_contents (terminal, stream,
-						     VTE_WRITE_DEFAULT,
-						     NULL, &error);
+			vte_terminal_write_contents_sync (terminal, stream,
+                                                          VTE_WRITE_DEFAULT,
+                                                          NULL, &error);
 			g_object_unref (stream);
 		}
 
@@ -944,7 +944,9 @@ main(int argc, char **argv)
 	}
 
         if (encoding != NULL) {
-                vte_terminal_set_encoding(terminal, encoding);
+                if (!vte_terminal_set_encoding(terminal, encoding, &error)) {
+                        g_printerr("Failed to set encoding: %s\n", error->message);
+                }
                 g_free(encoding);
         }
         if (cjk_ambiguous_width != NULL) {
