@@ -22,9 +22,12 @@
 #define vterowdata_h_included
 
 #include "vteunistr.h"
+#include "vtemacros.h"
 
 G_BEGIN_DECLS
 
+#define VTE_TAB_WIDTH_BITS		4  /* Has to be able to store the value of 8. */
+#define VTE_TAB_WIDTH_MAX		((1 << VTE_TAB_WIDTH_BITS) - 1)
 
 #define VTE_DEFAULT_FG			256
 #define VTE_DEFAULT_BG			257
@@ -35,15 +38,6 @@ G_BEGIN_DECLS
 #define VTE_CURSOR_BG			262
 #define VTE_PALETTE_SIZE		263
 
-
-#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 6)
-#define VTE_GNUC_PACKED \
-	__attribute__((__packed__))
-#else
-#define VTE_GNUC_PACKED
-#endif  /* !__GNUC__ */
-
-
 /*
  * VteCellAttr: A single cell style attributes
  *
@@ -55,18 +49,16 @@ G_BEGIN_DECLS
 
 typedef struct _VteCellAttr {
 	guint64 fragment: 1;	/* A continuation cell. */
-	guint64 columns: 4;	/* Number of visible columns
-				   (as determined by g_unicode_iswide(c)).
-				   Also abused for tabs; bug 353610
-				   Keep at least 4 for tabs to work
-				   */
+	guint64 columns: VTE_TAB_WIDTH_BITS;	/* Number of visible columns
+						   (as determined by g_unicode_iswide(c)).
+						   Also abused for tabs; bug 353610
+						   */
 	guint64 bold: 1;
 	guint64 italic: 1;
 	guint64 fore: 25;	/* Index into color palette, or direct RGB, */
 	/* 4-byte boundary */
 	guint64 back: 25;	/* see vte-private.h */
 
-	guint64 standout: 1;
 	guint64 underline: 1;
 	guint64 strikethrough: 1;
 
@@ -75,6 +67,7 @@ typedef struct _VteCellAttr {
 	guint64 half: 1;
 
 	guint64 invisible: 1;
+        /* 1 bit unused */
 } VteCellAttr;
 G_STATIC_ASSERT (sizeof (VteCellAttr) == 8);
 
@@ -88,7 +81,7 @@ G_STATIC_ASSERT (sizeof (VteCellAttr) == sizeof (VteIntCellAttr));
  * VteCell: A single cell's data
  */
 
-typedef struct VTE_GNUC_PACKED _VteCell {
+typedef struct _VTE_GNUC_PACKED _VteCell {
 	vteunistr c;
 	VteCellAttr attr;
 } VteCell;
@@ -96,7 +89,7 @@ G_STATIC_ASSERT (sizeof (VteCell) == 12);
 
 typedef union _VteIntCell {
 	VteCell cell;
-	struct VTE_GNUC_PACKED {
+	struct _VTE_GNUC_PACKED {
 		guint32 c;
 		guint64 attr;
 	} i;
@@ -114,7 +107,6 @@ static const VteIntCell basic_cell = {
 			VTE_DEFAULT_FG, /* fore */
 			VTE_DEFAULT_BG, /* back */
 
-			0, /* standout */
 			0, /* underline */
 			0, /* strikethrough */
 
