@@ -243,6 +243,28 @@ vte_terminal_emit_resize_window(VteTerminal *terminal,
 
 /* Some common functions */
 
+/* In Xterm, upon printing a character in the last column the cursor doesn't
+ * advance.  It's special cased that printing the following letter will first
+ * wrap to the next row.
+ *
+ * As a rule of thumb, escape sequences that move the cursor (e.g. cursor up)
+ * or immediately update the visible contents (e.g. clear in line) disable
+ * this special mode, whereas escape sequences with no immediate visible
+ * effect (e.g. color change) leave this special mode on.  There are
+ * exceptions of course (e.g. scroll up).
+ *
+ * In VTE, a different technical approach is used.  The cursor is advanced to
+ * the invisible column on the right, but it's set back to the visible
+ * rightmost column whenever necessary (that is, before handling any of the
+ * sequences that disable the special cased mode in xterm).  (Bug 731155.)
+ */
+static void
+_vte_terminal_ensure_cursor_is_onscreen (VteTerminal *terminal)
+{
+        if (G_UNLIKELY (terminal->pvt->screen->cursor_current.col >= terminal->pvt->column_count))
+                terminal->pvt->screen->cursor_current.col = terminal->pvt->column_count - 1;
+}
+
 static void
 _vte_terminal_home_cursor (VteTerminal *terminal)
 {
@@ -809,13 +831,13 @@ vte_sequence_handler_decset_internal(VteTerminal *terminal,
 				"Leaving application cursor mode.\n");
 		break;
 	case 3:
-                /* 3: DECCOLM set/reset to 132/80 columns mode */
+                /* 3: DECCOLM set/reset to 132/80 columns mode, clear screen and cursor home */
                 if (terminal->pvt->deccolm_mode) {
                         vte_terminal_emit_resize_window(terminal,
                                                         set ? 132 : 80,
                                                         terminal->pvt->row_count);
-                        /* Request a resize and redraw. */
-                        _vte_invalidate_all(terminal);
+                        _vte_terminal_clear_screen(terminal);
+                        _vte_terminal_home_cursor(terminal);
                 }
 		break;
 	case 5:
@@ -824,9 +846,7 @@ vte_sequence_handler_decset_internal(VteTerminal *terminal,
 		break;
 	case 6:
 		/* Reposition the cursor in its new home position. */
-		terminal->pvt->screen->cursor_current.col = 0;
-		terminal->pvt->screen->cursor_current.row =
-			terminal->pvt->screen->insert_delta;
+                _vte_terminal_home_cursor (terminal);
 		break;
 	case 47:
 	case 1047:
@@ -928,6 +948,8 @@ _vte_sequence_handler_cb (VteTerminal *terminal, GValueArray *params)
 	VteCell *pcell;
 	screen = terminal->pvt->screen;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	/* Get the data for the row which the cursor points to. */
 	rowdata = _vte_terminal_ensure_row(terminal);
         /* Clean up Tab/CJK fragments. */
@@ -962,6 +984,8 @@ _vte_sequence_handler_cd (VteTerminal *terminal, GValueArray *params)
 	glong i;
 	VteScreen *screen;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 	/* If the cursor is actually on the screen, clear the rest of the
 	 * row the cursor is on and all of the rows below the cursor. */
@@ -1019,6 +1043,8 @@ _vte_sequence_handler_ce (VteTerminal *terminal, GValueArray *params)
 	VteRowData *rowdata;
 	VteScreen *screen;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 	/* Get the data for the row which the cursor points to. */
 	rowdata = _vte_terminal_ensure_row(terminal);
@@ -1073,7 +1099,7 @@ static void
 vte_sequence_handler_cursor_position (VteTerminal *terminal, GValueArray *params)
 {
 	GValue *row, *col;
-	long rowval, colval, origin;
+        long rowval, colval, origin, rowmax;
 	VteScreen *screen;
 
 	screen = terminal->pvt->screen;
@@ -1087,11 +1113,13 @@ vte_sequence_handler_cursor_position (VteTerminal *terminal, GValueArray *params
 			if (screen->origin_mode &&
 			    screen->scrolling_restricted) {
 				origin = screen->scrolling_region.start;
+                                rowmax = screen->scrolling_region.end;
 			} else {
 				origin = 0;
+                                rowmax = terminal->pvt->row_count - 1;
 			}
                         rowval = g_value_get_long(row) - 1 + origin;
-			rowval = CLAMP(rowval, 0, terminal->pvt->row_count - 1);
+                        rowval = CLAMP(rowval, origin, rowmax);
 		}
 		if (params->n_values >= 2) {
 			col = g_value_array_get_nth(params, 1);
@@ -1191,24 +1219,29 @@ vte_sequence_handler_line_position_absolute (VteTerminal *terminal, GValueArray
 {
 	VteScreen *screen;
 	GValue *value;
-	long val, origin;
+        long val = 1, origin, rowmax;
 	screen = terminal->pvt->screen;
-	/* We only care if there's a parameter in there. */
+
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	if ((params != NULL) && (params->n_values > 0)) {
 		value = g_value_array_get_nth(params, 0);
 		if (G_VALUE_HOLDS_LONG(value)) {
-			/* Move the cursor. */
-			if (screen->origin_mode &&
-			    screen->scrolling_restricted) {
-				origin = screen->scrolling_region.start;
-			} else {
-				origin = 0;
-			}
-                        val = g_value_get_long(value) - 1 + origin;
-			val = CLAMP(val, 0, terminal->pvt->row_count - 1);
-			screen->cursor_current.row = screen->insert_delta + val;
+                        val = g_value_get_long(value);
 		}
 	}
+
+        if (screen->origin_mode &&
+            screen->scrolling_restricted) {
+                origin = screen->scrolling_region.start;
+                rowmax = screen->scrolling_region.end;
+        } else {
+                origin = 0;
+                rowmax = terminal->pvt->row_count - 1;
+        }
+        val = val - 1 + origin;
+        val = CLAMP(val, origin, rowmax);
+        screen->cursor_current.row = screen->insert_delta + val;
 }
 
 /* Delete a character at the current cursor position. */
@@ -1219,6 +1252,8 @@ _vte_sequence_handler_dc (VteTerminal *terminal, GValueArray *params)
 	VteRowData *rowdata;
 	long col;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 
 	if (_vte_ring_next(screen->row_data) > screen->cursor_current.row) {
@@ -1264,6 +1299,8 @@ vte_sequence_handler_cursor_down (VteTerminal *terminal, GValueArray *params)
         GValue *value;
         long val;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 
 	if (screen->scrolling_restricted) {
@@ -1295,6 +1332,8 @@ vte_sequence_handler_erase_characters (VteTerminal *terminal, GValueArray *param
 	VteCell *cell;
 	long col, i, count;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 
 	/* If we got a parameter, use it. */
@@ -1352,6 +1391,8 @@ _vte_sequence_handler_insert_character (VteTerminal *terminal, GValueArray *para
 	VteVisualPosition save;
 	VteScreen *screen;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 
 	save = screen->cursor_current;
@@ -1382,6 +1423,8 @@ vte_sequence_handler_backspace (VteTerminal *terminal, GValueArray *params)
 {
 	VteScreen *screen;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 	if (screen->cursor_current.col > 0) {
 		/* There's room to move left, so do so. */
@@ -1397,6 +1440,8 @@ vte_sequence_handler_cursor_backward (VteTerminal *terminal, GValueArray *params
         GValue *value;
         long val;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
         screen = terminal->pvt->screen;
 
         val = 1;
@@ -1417,6 +1462,8 @@ vte_sequence_handler_cursor_forward (VteTerminal *terminal, GValueArray *params)
         GValue *value;
         long val;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 
         val = 1;
@@ -1461,6 +1508,8 @@ vte_sequence_handler_restore_cursor (VteTerminal *terminal, GValueArray *params)
 					   screen->insert_delta,
 					   screen->insert_delta +
 					   terminal->pvt->row_count - 1);
+
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
 }
 
 /* Save cursor (position). */
@@ -1482,6 +1531,8 @@ vte_sequence_handler_scroll_down (VteTerminal *terminal, GValueArray *params)
 	long val = 1;
 	GValue *value;
 
+        /* No _vte_terminal_ensure_cursor_is_onscreen() here as per xterm */
+
 	if ((params != NULL) && (params->n_values > 0)) {
 		value = g_value_array_get_nth(params, 0);
 		if (G_VALUE_HOLDS_LONG(value)) {
@@ -1596,6 +1647,8 @@ vte_sequence_handler_scroll_up (VteTerminal *terminal, GValueArray *params)
 	long val = 1;
 	GValue *value;
 
+        /* No _vte_terminal_ensure_cursor_is_onscreen() here as per xterm */
+
 	if ((params != NULL) && (params->n_values > 0)) {
 		value = g_value_array_get_nth(params, 0);
 		if (G_VALUE_HOLDS_LONG(value)) {
@@ -1611,6 +1664,8 @@ vte_sequence_handler_scroll_up (VteTerminal *terminal, GValueArray *params)
 static void
 vte_sequence_handler_line_feed (VteTerminal *terminal, GValueArray *params)
 {
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	_vte_terminal_cursor_down (terminal);
 }
 
@@ -1621,6 +1676,8 @@ vte_sequence_handler_reverse_index (VteTerminal *terminal, GValueArray *params)
 	long start, end;
 	VteScreen *screen;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 
 	if (screen->scrolling_restricted) {
@@ -1790,6 +1847,8 @@ vte_sequence_handler_cursor_up (VteTerminal *terminal, GValueArray *params)
         GValue *value;
         long val;
 
+        _vte_terminal_ensure_cursor_is_onscreen(terminal);
+
 	screen = terminal->pvt->screen;
 
 	if (screen->scrolling_restricted) {
@@ -2266,23 +2325,11 @@ vte_sequence_handler_normal_keypad (VteTerminal *terminal, GValueArray *params)
 	terminal->pvt->keypad_mode = VTE_KEYMODE_NORMAL;
 }
 
-/* Move the cursor to the given column (horizontal position), 1-based. */
+/* Same as cursor_character_absolute, not widely supported. */
 static void
 vte_sequence_handler_character_position_absolute (VteTerminal *terminal, GValueArray *params)
 {
-        GValue *value;
-        long val;
-
-        /* We only care if there's a parameter in there. */
-        if ((params != NULL) && (params->n_values > 0)) {
-                value = g_value_array_get_nth(params, 0);
-                if (G_VALUE_HOLDS_LONG(value)) {
-                        val = g_value_get_long(value);
-                        if (val > 0) {
-                                vte_sequence_handler_cursor_character_absolute (terminal, params);
-                        }
-                }
-        }
+	vte_sequence_handler_cursor_character_absolute (terminal, params);
 }
 
 /* Set certain terminal attributes. */
@@ -2488,6 +2535,7 @@ vte_sequence_handler_insert_lines (VteTerminal *terminal, GValueArray *params)
 		_vte_terminal_ring_remove (terminal, end);
 		_vte_terminal_ring_insert (terminal, row, TRUE);
 	}
+        terminal->pvt->screen->cursor_current.col = 0;
 	/* Update the display. */
 	_vte_terminal_scroll_region(terminal, row, end - row + 1, param);
 	/* Adjust the scrollbars if necessary. */
@@ -2535,6 +2583,7 @@ vte_sequence_handler_delete_lines (VteTerminal *terminal, GValueArray *params)
 		_vte_terminal_ring_remove (terminal, row);
 		_vte_terminal_ring_insert (terminal, end, TRUE);
 	}
+        terminal->pvt->screen->cursor_current.col = 0;
 	/* Update the display. */
 	_vte_terminal_scroll_region(terminal, row, end - row + 1, -param);
 	/* Adjust the scrollbars if necessary. */
@@ -2549,13 +2598,13 @@ vte_sequence_handler_default_character_set (VteTerminal *terminal, GValueArray *
 {
 	G_CONST_RETURN char *locale_encoding;
 	g_get_charset(&locale_encoding);
-	vte_terminal_set_encoding(terminal, locale_encoding);
+	vte_terminal_set_encoding(terminal, locale_encoding, NULL);
 }
 
 static void
 vte_sequence_handler_utf_8_character_set (VteTerminal *terminal, GValueArray *params)
 {
-	vte_terminal_set_encoding(terminal, "UTF-8");
+	vte_terminal_set_encoding(terminal, NULL /* UTF-8 */, NULL);
 }
 
 /* Device status reports. The possible reports are the cursor position and
@@ -2565,7 +2614,7 @@ vte_sequence_handler_device_status_report (VteTerminal *terminal, GValueArray *p
 {
 	GValue *value;
 	VteScreen *screen;
-	long param;
+        long param, rowval, origin, rowmax;
 	char buf[128];
 
 	screen = terminal->pvt->screen;
@@ -2581,11 +2630,21 @@ vte_sequence_handler_device_status_report (VteTerminal *terminal, GValueArray *p
 				break;
 			case 6:
 				/* Send the cursor position. */
+                                if (screen->origin_mode &&
+                                    screen->scrolling_restricted) {
+                                        origin = screen->scrolling_region.start;
+                                        rowmax = screen->scrolling_region.end;
+                                } else {
+                                        origin = 0;
+                                        rowmax = terminal->pvt->row_count - 1;
+                                }
+                                rowval = screen->cursor_current.row - screen->insert_delta - origin;
+                                rowval = CLAMP(rowval, 0, rowmax);
 				g_snprintf(buf, sizeof(buf),
 					   _VTE_CAP_CSI "%ld;%ldR",
-					   screen->cursor_current.row + 1 -
-					   screen->insert_delta,
-					   screen->cursor_current.col + 1);
+                                           rowval + 1,
+                                           CLAMP(screen->cursor_current.col + 1,
+                                                 1, terminal->pvt->column_count));
 				vte_terminal_feed_child(terminal, buf, -1);
 				break;
 			default:
@@ -2601,7 +2660,7 @@ vte_sequence_handler_dec_device_status_report (VteTerminal *terminal, GValueArra
 {
 	GValue *value;
 	VteScreen *screen;
-	long param;
+        long param, rowval, origin, rowmax;
 	char buf[128];
 
 	screen = terminal->pvt->screen;
@@ -2613,11 +2672,21 @@ vte_sequence_handler_dec_device_status_report (VteTerminal *terminal, GValueArra
 			switch (param) {
 			case 6:
 				/* Send the cursor position. */
+                                if (screen->origin_mode &&
+                                    screen->scrolling_restricted) {
+                                        origin = screen->scrolling_region.start;
+                                        rowmax = screen->scrolling_region.end;
+                                } else {
+                                        origin = 0;
+                                        rowmax = terminal->pvt->row_count - 1;
+                                }
+                                rowval = screen->cursor_current.row - screen->insert_delta - origin;
+                                rowval = CLAMP(rowval, 0, rowmax);
 				g_snprintf(buf, sizeof(buf),
 					   _VTE_CAP_CSI "?%ld;%ldR",
-					   screen->cursor_current.row + 1 -
-					   screen->insert_delta,
-					   screen->cursor_current.col + 1);
+                                           rowval + 1,
+                                           CLAMP(screen->cursor_current.col + 1,
+                                                 1, terminal->pvt->column_count));
 				vte_terminal_feed_child(terminal, buf, -1);
 				break;
 			case 15:
