@@ -323,8 +323,9 @@ gboolean  vte_terminal_search_find_next       (VteTerminal *terminal) _VTE_GNUC_
 
 
 /* Set the character encoding.  Most of the time you won't need this. */
-void vte_terminal_set_encoding(VteTerminal *terminal,
-                               const char *codeset) _VTE_GNUC_NONNULL(1);
+gboolean vte_terminal_set_encoding(VteTerminal *terminal,
+                                   const char *codeset,
+                                   GError **error) _VTE_GNUC_NONNULL(1);
 const char *vte_terminal_get_encoding(VteTerminal *terminal) _VTE_GNUC_NONNULL(1);
 
 /* CJK compatibility setting */
@@ -360,11 +361,11 @@ void vte_terminal_set_geometry_hints_for_window(VteTerminal *terminal,
                                                 GtkWindow *window) _VTE_GNUC_NONNULL(1) _VTE_GNUC_NONNULL(2);
 
 /* Writing contents out */
-gboolean vte_terminal_write_contents (VteTerminal *terminal,
-				      GOutputStream *stream,
-				      VteWriteFlags flags,
-				      GCancellable *cancellable,
-				      GError **error) _VTE_GNUC_NONNULL(1) _VTE_GNUC_NONNULL(2);
+gboolean vte_terminal_write_contents_sync (VteTerminal *terminal,
+                                           GOutputStream *stream,
+                                           VteWriteFlags flags,
+                                           GCancellable *cancellable,
+                                           GError **error) _VTE_GNUC_NONNULL(1) _VTE_GNUC_NONNULL(2);
 
 G_END_DECLS
 
